from consts import *
import threading
import socket
import time
import xbmc
import traceback


def parse_episode(episode_id="0", first="0"):
    first = 0
    if first == "1":
        show_busy_dialog()

    url = None
    url_response = __api__.__get__("get/file", __data__={
        "id": episode_id
    })

    if url_response:
        url = url_response["data"]

    output("[EPISODE] URL %s" % url)

    if not url:
        return None

    if first == "1":
        hide_busy_dialog()

    return url


class __PlayServer__(object):
    def __init__(self):
        self._thread = None
        self._player = None
        self._running = False
        self._port = None
        self._episode_id = 0
        self._started = False
        self._last_progress = None
        self._exit_requested = False
        self._running_since = 0
        self._times = {}
        self._player = xbmc.Player()

    def __play_start__(self, __episode_id__):
        self._started = True
        output("[TRACK-" + str(self._port) + "] START %s" % __episode_id__)

        self._times[__episode_id__] = {
            "current": 0,
            "total": 0
        }

        self.__play_progress__(__episode_id__)

    def __play_progress__(self, __episode_id__):
        if not __episode_id__ or not (__episode_id__ in self._times):
            return

        current = self._times[__episode_id__]
        current_time = current["current"]

        output("[TRACK-" + str(self._port) + "] PROGRESS %s - %s" % (__episode_id__, current_time))

        save_watchtime(__episode_id__, current_time, False)

    def __play_end__(self, __episode_id__):
        self._started = False

        if not __episode_id__ or not (__episode_id__ in self._times):
            return

        current = self._times[__episode_id__]
        current_time = current["current"]
        total_time = current["total"]
        seen = total_time > 0.0 and int(abs(total_time - current_time)) < (10 * 60)

        output("[TRACK-" + str(self._port) + "] END %s - %s | seen: %s" % (__episode_id__, current_time, seen))

        del self._times[__episode_id__]

        save_watchtime(__episode_id__, current_time, seen)

    def __handle__(self, __client_socket__, __placeholder__=None):
        output("[SERVER-" + str(self._port) + "] Handle client")

        try:
            xbmc.sleep(50)

            request = __client_socket__.recv(1024)

            if not request:
                return False

            matches = re.search(r"^([^\s]+) /([^/]+)(?:/([^/]+))?/[^\s]* HTTP", request, re.IGNORECASE | re.MULTILINE)
            if matches:
                output("[SERVER-" + str(self._port) + "] Request: %s" % str([matches.group(i + 1) for i in xrange(len(matches.groups()))]))

                method = matches.group(1)
                command = matches.group(2)

                if method == "HEAD":
                    if command == "play":
                        __client_socket__.send("HTTP/1.1 200 OK\r\nContent-Type: application/octet-stream\r\nConnection: close\r\n\r\n")

                else:
                    if command == "exit":
                        __client_socket__.send("HTTP/1.1 200 OK\r\nContent-Length: 2\r\nContent-Type: text/plain\r\nConnection: close\r\n\r\nOK")
                        self._exit_requested = True
                        output("[SERVER-" + str(self._port) + "] Playserver will be closed")
                        self._running = False
                        return True

                    elif command == "play":
                        read_settings()
                        check_settings()

                        episode_id = matches.group(3)

                        self.__play_end__(self._episode_id)

                        show_busy_dialog()
                        try:
                            episode_url = parse_episode(episode_id, "0")

                            if not episode_url:
                                raise Exception("Got no episode url")

                            self._episode_id = episode_id
                            __client_socket__.send("HTTP/1.1 301 Moved Permanently\r\nLocation: %s\r\nConnection: close\r\n\r\n" % episode_url)

                        except Exception as e:
                            self._running = False
                            output("[SERVER-" + str(self._port) + "] Client Play Error: %s" % traceback.format_exc())
                            __client_socket__.send("HTTP/1.1 400 Bad Request\r\nConnection: close\r\n\r\n")

                            PLUGIN.notify("Fehler beim Abspielen!")

                        finally:
                            hide_busy_dialog()

                    elif command == "reset":
                        pass

                    else:
                        output("[SERVER-" + str(self._port) + "] Unknown command: %s" % command)
            else:
                output("[SERVER-" + str(self._port) + "] Wrong client Request %s" % request)

            return True

        except:
            self._running = False
            output("[SERVER-" + str(self._port) + "] Client Error: %s" % traceback.format_exc())
            return False

        finally:
            __client_socket__.close()

    def __cleanup__(self):
        self._thread = None
        self._running = False
        self._port = None
        self._player = None
        self._episode_id = 0
        self._started = False
        self._last_progress = None
        self._times = {}

    def __track__(self, aborted):
        if not self._episode_id:
            return

        def do_end():
            if self._started:
                self.__play_end__(self._episode_id)
                self._episode_id = 0
            self._started = False

        is_playing = self._player.isPlayingVideo() and not aborted

        if is_playing:
            current_file = self._player.getPlayingFile()

            if "127.0.0.1:%s/play/%s/" % (self._port, self._episode_id) in current_file:
                if not self._started:
                    self._last_progress = 0

                    self.__play_start__(self._episode_id)

                else:
                    if self._times[self._episode_id]["total"] <= 0:
                        self._times[self._episode_id]["total"] = int(self._player.getTotalTime())

                    # output("PROGRESS %s" % current_time)

                    current_time = int(self._player.getTime())
                    current_time = max(0, current_time - 5)

                    self._times[self._episode_id]["current"] = current_time

                    if time.time() - self._last_progress >= 15:
                        self._last_progress = time.time()

                        self.__play_progress__(self._episode_id)

            elif self._started:
                do_end()

        else:
            do_end()

    @staticmethod
    def __save__(port, created):
        server_storage = get_storage("server")
        server_storage["created"] = created
        server_storage["port"] = port
        server_storage.sync()

    def __callback__(self):
        server = None
        monitor = None

        port = None

        try:
            server_storage = get_storage("server")
            if not (server_storage is None or not ("port" in server_storage)):
                created = int(server_storage["created"])
                port = int(server_storage["port"])

                if not port or time.time() - created > 60 * 60:
                    port = None
                    self.__save__(0, 0)

        except:
            pass

        if port:
            try:
                # Try to reach current running Server
                if socket.create_connection(("127.0.0.1", port), timeout=5):
                    self._port = port

                    output("[SERVER-" + str(self._port) + "] Playserver reached at %s" % port)

                    return True

                raise Exception("")

            except:
                port = None
                output("[SERVER-" + str(self._port) + "] Playserver not reachable")

        output("[SERVER-" + str(self._port) + "] Need to create Playserver")

        last_file = None
        try:
            server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            try:
                server.bind(("127.0.0.1", 0))
            except Exception as e:
                output("[SERVER-" + str(self._port) + "] Startup Error: %s" % traceback.format_exc())

                return False

            # Setup socket
            try:
                server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            except:
                output("[SERVER-" + str(self._port) + "] Failed to set SO_REUSEADDR: %s" % traceback.format_exc())

            try:
                server.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
            except:
                output("[SERVER-" + str(self._port) + "] Failed to set TCP_NODELAY: %s" % traceback.format_exc())

            server.listen(5)

            addr = server.getsockname()
            self._running = True
            self._port = addr[1]

            self.__save__(self._port, time.time())

            output("[SERVER-" + str(self._port) + "] Started")

            failed = 0
            monitor = xbmc.Monitor()
            while True:
                client_sock = None
                address = None

                try:
                    server.settimeout(1)
                    client_sock, address = server.accept()
                except socket.timeout:
                    pass
                except Exception as e:
                    output("[SERVER-" + str(self._port) + "] Error: %s" % traceback.format_exc())
                    failed += 1
                    if failed > 10:
                        self.__save__(0, 0)
                        raise Exception("Too many errors")

                    xbmc.sleep(100)
                    continue

                failed = 0

                if client_sock:
                    output("[SERVER-" + str(self._port) + "] Got client! %s" % str(address))

                    try:
                        client_handler = threading.Thread(
                            target=self.__handle__,
                            args=(client_sock, None)
                        )
                        client_handler.start()

                        output("[SERVER-" + str(self._port) + "] Client thread started!")

                    except Exception as e:
                        output("[SERVER-" + str(self._port) + "] Error: %s" % traceback.format_exc())
                        client_sock.close()

                current_file = None
                is_playing = self._player.isPlayingVideo()
                if is_playing:
                    current_file = self._player.getPlayingFile()

                aborted = monitor.abortRequested() > 0 or not self._running or self._exit_requested
                output("[SERVER-" + str(self._port) + "] Idle abort: %s / file: %s" % (aborted, current_file))

                self.__track__(aborted)
                if aborted:
                    break

            return True

        except Exception as e:
            output("[SERVER-" + str(self._port) + "] FUCKOFF: %s" % traceback.format_exc())

            return False

        finally:
            if server:
                output("[SERVER-" + str(self._port) + "] Closed")

            is_playing = self._player.isPlayingVideo()
            current_file = None
            if is_playing:
                current_file = self._player.getPlayingFile()
                if current_file and "127.0.0.1:%s/play/" % self._port in current_file:
                    output("[SERVER-" + str(self._port) + "] Also stop Player")

                    xbmc.executebuiltin("PlayerControl(stop)")

                else:
                    current_file = None

            if self._exit_requested or DEBUG:
                PLUGIN.notify("Playserver wurde beendet")

            try:
                if server:
                    server.close()
            except:
                pass

            monitor = None
            self.__save__(0, 0)
            self.__cleanup__()

    @property
    def __is_running__(self):
        return self._running

    @property
    def __port__(self):
        return self._port

    def __start__(self):
        self._thread = threading.Thread(target=self.__callback__)
        self._thread.start()
